"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUpdateChoices = void 0;
const colorize_semver_diff_1 = __importDefault(require("@pnpm/colorize-semver-diff"));
const semver_diff_1 = __importDefault(require("@pnpm/semver-diff"));
const table_1 = require("@zkochan/table");
const ramda_1 = require("ramda");
const isEmpty_1 = __importDefault(require("ramda/src/isEmpty"));
function getUpdateChoices(outdatedPkgsOfProjects, workspacesEnabled) {
    if ((0, isEmpty_1.default)(outdatedPkgsOfProjects)) {
        return [];
    }
    const pkgUniqueKey = (outdatedPkg) => {
        return JSON.stringify([outdatedPkg.packageName, outdatedPkg.latestManifest?.version, outdatedPkg.current]);
    };
    const dedupeAndGroupPkgs = (0, ramda_1.pipe)((0, ramda_1.uniqBy)((outdatedPkg) => pkgUniqueKey(outdatedPkg)), (0, ramda_1.groupBy)((outdatedPkg) => outdatedPkg.belongsTo));
    const groupPkgsByType = dedupeAndGroupPkgs(outdatedPkgsOfProjects);
    const headerRow = {
        Package: true,
        Current: true,
        ' ': true,
        Target: true,
        Workspace: workspacesEnabled,
        URL: true,
    };
    // returns only the keys that are true
    const header = Object.keys((0, ramda_1.pickBy)(ramda_1.and, headerRow));
    return Object.entries(groupPkgsByType).reduce((finalChoices, [depGroup, choiceRows]) => {
        if (choiceRows.length === 0) {
            return finalChoices;
        }
        const rawChoices = choiceRows.map(choice => buildPkgChoice(choice, workspacesEnabled));
        // add in a header row for each group
        rawChoices.unshift({
            raw: header,
            name: '',
            disabled: true,
        });
        const renderedTable = alignColumns((0, ramda_1.pluck)('raw', rawChoices)).filter(Boolean);
        const choices = rawChoices.map((outdatedPkg, i) => {
            if (i === 0) {
                return {
                    name: renderedTable[i],
                    value: '',
                    disabled: true,
                    hint: '',
                };
            }
            return {
                name: outdatedPkg.name,
                message: renderedTable[i],
                value: outdatedPkg.name,
            };
        });
        // To filter out selected "dependencies" or "devDependencies" in the final output,
        // we rename it here to "[dependencies]" or "[devDependencies]",
        // which will be filtered out in the format function of the prompt.
        finalChoices.push({ name: `[${depGroup}]`, choices, message: depGroup });
        return finalChoices;
    }, []);
}
exports.getUpdateChoices = getUpdateChoices;
function buildPkgChoice(outdatedPkg, workspacesEnabled) {
    const sdiff = (0, semver_diff_1.default)(outdatedPkg.wanted, outdatedPkg.latestManifest.version);
    const nextVersion = sdiff.change === null
        ? outdatedPkg.latestManifest.version
        : (0, colorize_semver_diff_1.default)(sdiff); // eslint-disable-line @typescript-eslint/no-explicit-any
    const label = outdatedPkg.packageName;
    const lineParts = {
        label,
        current: outdatedPkg.current,
        arrow: '❯',
        nextVersion,
        workspace: outdatedPkg.workspace,
        url: getPkgUrl(outdatedPkg),
    };
    if (!workspacesEnabled) {
        delete lineParts.workspace;
    }
    return {
        raw: Object.values(lineParts),
        name: outdatedPkg.packageName,
    };
}
function getPkgUrl(pkg) {
    if (pkg.latestManifest?.homepage) {
        return pkg.latestManifest?.homepage;
    }
    if (typeof pkg.latestManifest?.repository !== 'string') {
        if (pkg.latestManifest?.repository?.url) {
            return pkg.latestManifest?.repository?.url;
        }
    }
    return '';
}
function alignColumns(rows) {
    return (0, table_1.table)(rows, {
        border: (0, table_1.getBorderCharacters)('void'),
        columnDefault: {
            paddingLeft: 0,
            paddingRight: 1,
            wrapWord: true,
        },
        columns: {
            0: { width: 50, truncate: 100 },
            1: { width: 15, alignment: 'right' },
            3: { width: 15 },
            4: { paddingLeft: 2 },
            5: { paddingLeft: 2 },
        },
        drawHorizontalLine: () => false,
    }).split('\n');
}
//# sourceMappingURL=getUpdateChoices.js.map